"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.replicateEvents = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

var _crc = require("crc-32");

const replicateEvents = async (pool, events) => {
  if (events.length === 0) return;
  const {
    executeStatement,
    executeQuery,
    eventsTableName,
    escape,
    escapeId
  } = pool;
  const eventsTableNameAsId = escapeId(eventsTableName);
  const rows = await executeStatement(`SELECT "threadId", MAX("threadCounter") AS "threadCounter" FROM 
    ${eventsTableNameAsId} GROUP BY "threadId" ORDER BY "threadId" ASC`);
  const threadCounters = new Array(_eventstoreBase.THREAD_COUNT);

  for (const row of rows) {
    threadCounters[row.threadId] = +row.threadCounter;
  }

  const eventsToInsert = [];

  for (const event of events) {
    const threadId = Math.abs((0, _crc.str)(event.aggregateId + event.aggregateVersion)) % threadCounters.length;
    const threadCounter = threadCounters[threadId] === undefined ? 0 : threadCounters[threadId] + 1;
    threadCounters[threadId] = threadCounter;
    eventsToInsert.push({ ...event,
      threadId,
      threadCounter
    });
  }

  await executeQuery(`INSERT OR IGNORE INTO ${eventsTableNameAsId}(
      "threadId",
      "threadCounter",
      "timestamp",
      "aggregateId",
      "aggregateVersion",
      "type",
      "payload"
    ) VALUES ${eventsToInsert.map(event => `(
      ${+event.threadId},
      ${+event.threadCounter},
      ${+event.timestamp},
      ${escape(event.aggregateId)},
      ${+event.aggregateVersion},
      ${escape(event.type)},
      json(CAST(${event.payload != null ? escape(JSON.stringify(event.payload)) : escape('null')} AS BLOB))
    )`).join(',')}`);
};

exports.replicateEvents = replicateEvents;
var _default = replicateEvents;
exports.default = _default;
//# sourceMappingURL=replicate-events.js.map